@@ -1,3 +1,4 @@
+from abc import ABCMeta
 from python_helper import Constant as c
 from python_helper import log, ReflectionHelper, ObjectHelper, StringHelper
 
@@ -10,36 +11,85 @@ ENUM_STATIC_METHOD_NAME_LIST = [
     MAP_METHOD_NAME
 ]
 
-class EnumItemStr(str) :
-    ...
-class EnumItemInt(int) :
-    ...
-class EnumItemFloat(float) :
-    ...
-class EnumItemDict(dict) :
-    ...
-class EnumItemSet(set) :
-    ...
-class EnumItemTuple(tuple) :
-    ...
-class EnumItemList(list) :
-    ...
+def isEnumItemInstance(instance):
+    return EnumItem == type(instance)
 
-class EnumClass(object) :
-    ...
+def areUnsafellyEnumItemInstance(one, theOther):
+    return type(one) == type(theOther)
 
-def updateEnumItem(enumItem, key, value) :
+def areEnumItemInstance(one, theOther):
+    return isEnumItemInstance(one) and areUnsafellyEnumItemInstance(one, theOther)
+
+def mustUpdateInnerAttributeValueType(possibleEnumInnerValueType, value):
+    return ObjectHelper.isNotNone(possibleEnumInnerValueType) and value not in DERIVATED_ENUM_ITEM_CLASS_SET
+
+def updateEnumItem(enumItem, key, value):
     enumItem.__enumItemMap__[key] = value
     enumItem.__enumItemEqMap__[key] = str(value)
-    ReflectionHelper.setAttributeOrMethod(enumItem, key, ENUM_ITEM_CLASS_DICTIONARY.get(type(value))(value))
+    setInnerAttributeValue(enumItem, key, value)
+
+def setInnerAttributeValue(instance, attributeName, value):
+    EnumInnerValueType = ENUM_ITEM_CLASS_DICTIONARY.get(type(value))
+    # print(f'    -> {value=}')
+    # print(f'    -> {EnumInnerValueType=}')
+    if mustUpdateInnerAttributeValueType(EnumInnerValueType, value):
+        ReflectionHelper.setAttributeOrMethod(instance, attributeName, EnumInnerValueType(value))
+    else:
+        ReflectionHelper.setAttributeOrMethod(instance, attributeName, value)
+    # print(f'ReflectionHelper.getAttributeOrMethod({instance}, {attributeName}, {value}): {ReflectionHelper.getAttributeOrMethod(instance, attributeName)}')
+
+
+class EnumItemStr(str):
+    ...
+class EnumItemInt(int):
+    ...
+class EnumItemFloat(float):
+    ...
+class EnumItemDict(dict):
+    def __instancecheck__(self, instance):
+        return type(instance) in DERIVATED_ENUM_ITEM_CLASS_SET
+    def __subclasscheck__(self, instance):
+        return type(instance) == EnumItem
+    ...
+class EnumItemSet(set):
+    def __instancecheck__(self, instance):
+        return type(instance) in DERIVATED_ENUM_ITEM_CLASS_SET
+    def __subclasscheck__(self, instance):
+        return type(instance) == EnumItem
+    ...
+class EnumItemTuple(tuple):
+    def __instancecheck__(self, instance):
+        return type(instance) in DERIVATED_ENUM_ITEM_CLASS_SET
+    def __subclasscheck__(self, instance):
+        return type(instance) == EnumItem
+    ...
+class EnumItemList(list):
+    def __instancecheck__(self, instance):
+        return type(instance) in DERIVATED_ENUM_ITEM_CLASS_SET
+    def __subclasscheck__(self, instance):
+        return type(instance) == EnumItem
+    ...
+
+class EnumClass(object):
+    ...
+
+class EnumItem(metaclass=ABCMeta):
+
+    ###- __metaclass__ = ABCMeta
 
-class EnumItem :
     def __init__(self, *args, **kwargs):
+        # print(f'__init__({self}, *{args}, **{kwargs})')
+        # print(f'    type({self}): {type(self)}')
+        # print(f'    !---!---> isinstance({self}, EnumItem): {isinstance(self, EnumItem)}')
+        ###- if type(self) not in STRICTLY_DERIVATED_ENUM_ITEM_CLASS_SET:
         self.__enumItemMap__ = {}
         self.__enumItemEqMap__ = {}
         for key, value in kwargs.items():
             updateEnumItem(self, key, value)
 
+    def isInstance(self, other, givenClass):
+        return areUnsafellyEnumItemInstance(self, other)
+
     def __get__(self, obj, objtype=None):
         return self if not ReflectionHelper.hasAttributeOrMethod(self, ENUM_VALUE_AS_STRING_KEY) else self.enumValue
 
@@ -50,22 +100,22 @@ class EnumItem :
         return self.__str__()
 
     def __eq__(self, other):
-        if isinstance(other, EnumItem):
+        if self.isInstance(other, EnumItem):
             return str(other.__enumItemEqMap__) == str(self.__enumItemEqMap__)
         return self.enumValue == other
 
     def __ne__(self, other):
-        if isinstance(other, EnumItem):
+        if self.isInstance(other, EnumItem):
             return str(other.__enumItemEqMap__) != str(self.__enumItemEqMap__)
         return self.enumValue != other
 
     def __lt__(self, other):
-        if isinstance(other, EnumItem) and not isinstance(self.enumValue, EnumItem):
+        if self.isInstance(other, EnumItem) and not self.isInstance(self.enumValue, EnumItem):
             return self.enumValue < other.enumValue
         return self < other
 
     def __gt__(self, other):
-        if isinstance(other, EnumItem) and not isinstance(self.enumValue, EnumItem):
+        if self.isInstance(other, EnumItem) and not self.isInstance(self.enumValue, EnumItem):
             return self.enumValue > other.enumValue
 
     def __le__(self, other):
@@ -74,6 +124,21 @@ class EnumItem :
     def __ge__(self, other):
         return self.__eq__(other) or self.__gt__(other)
 
+    def __hash__(self):
+        return hash(self.__str__())
+
+    ###- def __instancecheck__(self, instance):
+    ###-     return type(instance) in DERIVATED_ENUM_ITEM_CLASS_SET
+    ###- def __subclasscheck__(self, instance):
+    ###-     return type(instance) == EnumItem
+
+EnumItem.register(EnumItemStr)
+EnumItem.register(EnumItemInt)
+EnumItem.register(EnumItemFloat)
+EnumItem.register(EnumItemDict)
+EnumItem.register(EnumItemSet)
+EnumItem.register(EnumItemTuple)
+EnumItem.register(EnumItemList)
 
 ENUM_ITEM_CLASS_DICTIONARY = {
     str : EnumItemStr,
@@ -93,7 +158,22 @@ ENUM_ITEM_CLASS_DICTIONARY = {
     EnumItem : EnumItem
 }
 
-def isEnum(possibleEnum) :
+STRICTLY_DERIVATED_ENUM_ITEM_CLASS_SET = {
+    EnumItemStr,
+    EnumItemInt,
+    EnumItemFloat,
+    EnumItemDict,
+    EnumItemSet,
+    EnumItemTuple,
+    EnumItemList
+}
+
+DERIVATED_ENUM_ITEM_CLASS_SET = set([
+    EnumItem,
+    *STRICTLY_DERIVATED_ENUM_ITEM_CLASS_SET
+])
+
+def isEnum(possibleEnum):
     itIs = False
     try :
         itIs = isinstance(possibleEnum, EnumItem)
@@ -101,7 +181,7 @@ def isEnum(possibleEnum) :
         log.error(isEnum, f'Not possible to evaluate. Returning {itIs} by default', exception)
     return itIs
 
-def isEnumItem(possibleEnumItem) :
+def isEnumItem(possibleEnumItem):
     itIs = False
     try :
         itIs = isinstance(possibleEnumItem, EnumItem)
@@ -109,11 +189,11 @@ def isEnumItem(possibleEnumItem) :
         log.error(isEnumItem, f'Not possible to evaluate. Returning {itIs} by default', exception)
     return itIs
 
-def Enum(instanceLog=False, associateReturnsTo=ENUM_VALUE_AS_STRING_KEY) :
+def Enum(instanceLog=False, associateReturnsTo=ENUM_VALUE_AS_STRING_KEY):
     def Wrapper(OuterEnum, *args, **kwargs):
-        def __raiseBadImplementation__(enumValue, enum=None) :
+        def __raiseBadImplementation__(enumValue, enum=None):
             raise Exception(f'Not possible to implement "{enumValue}" enum value in {enum} enum')
-        def __raiseEnumValueNotImplemented__(enumValue, enumClass=None, enumEqList=None, enumMap=None) :
+        def __raiseEnumValueNotImplemented__(enumValue, enumClass=None, enumEqList=None, enumMap=None):
             raise Exception(f'Not possible to retrieve "{enumValue}" of type: {type(enumValue)} enum value from {ReflectionHelper.getName(enumClass)}: {enumEqList} enum. Enum.__enumMap__ = {enumMap}')
         if instanceLog :
             log.log(Enum,f'''wrapping {OuterEnum.__name__}''')
@@ -138,23 +218,28 @@ def Enum(instanceLog=False, associateReturnsTo=ENUM_VALUE_AS_STRING_KEY) :
                         log.prettyPython(OuterEnum, 'attributeAttributeDataList', attributeAttributeDataList)
                         # print(f'type({attribute}): {type(attribute)}')
                     association = ReflectionHelper.getAttributeOrMethod(attribute, self.__associateReturnsTo__, muteLogs=True)
-                    if type(association) is type(None) :
+                    # if type(association) is type(None):
+                    if ObjectHelper.isNone(association):
                         # print('type(association) is type(None)')
-                        ReflectionHelper.setAttributeOrMethod(attribute, ENUM_VALUE_AS_STRING_KEY, ENUM_ITEM_CLASS_DICTIONARY.get(type(enumValue))(enumValue))
+                        setInnerAttributeValue(attribute, ENUM_VALUE_AS_STRING_KEY, enumValue)
                         nativeClassDataList = ReflectionHelper.getAttributeDataList(type(enumValue)())
                         # print(f'type({enumValue}): {type(enumValue)}, type({type(enumValue)}()): {type(type(enumValue)())}')
                     else :
                         # print('not type(association) is type(None)')
-                        ReflectionHelper.setAttributeOrMethod(attribute, ENUM_VALUE_AS_STRING_KEY, ENUM_ITEM_CLASS_DICTIONARY.get(type(association))(association))
+                        setInnerAttributeValue(attribute, ENUM_VALUE_AS_STRING_KEY, association)
                         nativeClassDataList = ReflectionHelper.getAttributeDataList(type(association)())
                         # print(f'type({association}): {type(association)}, type({type(association)}()): {type(type(association)())}')
                     # print()
                     attribute.enumValue.enumName = enumValue
                     ReflectionHelper.setAttributeOrMethod(attribute, ENUM_NAME_AS_STRING_KEY, enumValue)
-                    if isinstance(attribute, EnumItem) :
+                    # print(f'    ---> {attribute.enumValue.enumName=}')
+                    # print(f'    ---> {attribute.enumValue=}')
+                    # print(f'    ---> {attribute.enumName=}')
+                    # if isinstance(attribute, EnumItem) and type(attribute) not in STRICTLY_DERIVATED_ENUM_ITEM_CLASS_SET:
+                    if isEnumItemInstance(attribute) and type(attribute):
                         # print(attribute, attribute.enumName, attribute)
                         updateEnumItem(attribute, attribute.enumName, attribute) ###- so that __eq__ does not crash
-                    if not c.TYPE_FUNCTION == ReflectionHelper.getClassName(attribute) :
+                    if not c.TYPE_FUNCTION == ReflectionHelper.getClassName(attribute):
                         attribute.enumValue.__enumItemMap__ = attribute.__enumItemMap__
                         attribute.enumValue.__enumItemEqMap__ = attribute.__enumItemEqMap__
                     for dataInstance, dataName in attributeAttributeDataList :
@@ -168,11 +253,12 @@ def Enum(instanceLog=False, associateReturnsTo=ENUM_VALUE_AS_STRING_KEY) :
                     ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_VALUE_AS_STRING_KEY, ReflectionHelper.getAttributeOrMethod(attribute, self.__associateReturnsTo__, muteLogs=True))
                     ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_NAME_AS_STRING_KEY, enumValue)
                     ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_AS_STRING_KEY, self)
-                if ObjectHelper.isEmpty(OuterEnum.__enumEqList__) :
+                if ObjectHelper.isEmpty(OuterEnum.__enumEqList__):
                     for attribute, value in attributeDataList :
                         # print(f'type({value}): {type(value)}')
                         # print(f'type({attribute}): {type(attribute)}')
                         valueAsString = str(attribute.enumValue)
+                        # print(f'    --> {valueAsString=}')
                         if valueAsString not in ENUM_STATIC_METHOD_NAME_LIST :
                             if valueAsString not in self.__enumMap__ :
                                 self.__enumMap__[valueAsString] = attribute.enumValue ###- ReflectionHelper.getAttributeOrMethod(attribute, valueAsString, muteLogs=True) ###- attribute
@@ -184,23 +270,23 @@ def Enum(instanceLog=False, associateReturnsTo=ENUM_VALUE_AS_STRING_KEY) :
                                 __raiseBadImplementation__(valueAsString, enum=self)
                             self.__enumEqList__.append(f'{value}({self.__associateReturnsTo__}:{valueAsString})')
 
-            def __eq__(self, other) :
+            def __eq__(self, other):
                 return str(self) == str(other)
 
-            def __ne__(self, other) :
+            def __ne__(self, other):
                 return str(self) != str(other)
 
-            def __str__(self) :
+            def __str__(self):
                 return StringHelper.join([ReflectionHelper.getName(OuterEnum), c.COLON_SPACE, str(self.__enumEqList__)])
 
-            def __repr__(self) :
+            def __repr__(self):
                 return self.__str__()
 
             @staticmethod
-            def map(enumItemOrEnumItemValue) :
+            def map(enumItemOrEnumItemValue):
                 if instanceLog :
                     log.log(OuterEnum,f'''enumItemOrEnumItemValue: {enumItemOrEnumItemValue}''')
-                if ObjectHelper.isNone(enumItemOrEnumItemValue) :
+                if ObjectHelper.isNone(enumItemOrEnumItemValue):
                     return enumItemOrEnumItemValue
                 else :
                     # try :
